
local animation = {}
animation.__index = animation

function animation.new(texture, fps, frameStart, frameEnd, frameWidth, frameHeight, offsetX, offsetY)
	local self = setmetatable({}, animation)

	offsetX = offsetX or 0
	offsetY = offsetY or 0

	self._fps = fps
	self._spf = 1.0 / fps

	self._frameTimer = self._spf

    self._frameWidth = frameWidth
    self._frameHeight = frameHeight or frameWidth
	self._textureWidth, self._textureHeight = texture:getDimensions()

	self._frameStart = frameStart
	self._frameEnd = frameEnd

	self._currentFrame = frameStart
	self._done = false

	self.wrap = true
	self.speed = 1

	self.frames = {}

    for frame = self._frameStart, self._frameEnd do
        local w = self._textureWidth / self._frameWidth

        local x = frame % w * self._frameWidth
        local y = math.floor(frame / w) * self._frameHeight

        self.frames[frame] = love.graphics.newQuad(x + offsetX, y + offsetY, self._frameWidth, self._frameHeight, self._textureWidth, self._textureHeight)
    end

	return self
end

function animation:reset()
	self._frameTimer = self._spf
	self._currentFrame = self._frameStart
	self._done = false
end

function animation:done()
	return self._done
end

function animation:update(dt)
	self._frameTimer = self._frameTimer - dt*self.speed

	while self._frameTimer < 0 do
		self._frameTimer = self._frameTimer + self._spf

		self._currentFrame = self._currentFrame + 1
		if self._currentFrame > self._frameEnd then
			if self.wrap then
				self._currentFrame = self._frameStart
			else
				self._currentFrame = self._frameEnd
				self._done = true
			end
		end
	end

	while self._frameTimer > self._spf do
		self._frameTimer = self._frameTimer - self._spf

		self._currentFrame = self._currentFrame - 1
		if self._currentFrame < self._frameStart then
			if self.wrap then
				self._currentFrame = self._frameEnd
			else
				self._currentFrame = self._frameStart
				self._done = true
			end
		end
	end
end

function animation:getFrame()
	return self.frames[self._currentFrame]
end

function animation:getDimensions()
	local x, y, w, h = self:getFrame():getViewport()
	return w, h
end

return animation
