
local charactermanager = {}

local characters = {}
local players = {}

local basefactory = require("entities/characters/character")
local factories = {
	pirate = require("entities/characters/pirate"),
	sailor = require("entities/characters/sailor"),
	bunny = require("entities/characters/bunny"),
	mermaid = require("entities/characters/mermaid"),
}

local default_character = {"pirate", "sailor", "bunny", "mermaid"}

local icons_image = love.graphics.newImage("img/character-icon.png")
icons_image:setFilter("nearest", "nearest")

local icons = {
	love.graphics.newQuad(0, 0, 24, 24, 96, 24),
	love.graphics.newQuad(24, 0, 24, 24, 96, 24),
	love.graphics.newQuad(48, 0, 24, 24, 96, 24),
	love.graphics.newQuad(72, 0, 24, 24, 96, 24),
}

function charactermanager.initialize()
	for i, character in ipairs(default_character) do
		factory = factories[character]
		local newcharacter = {
			name = "character" .. i,
			icons_image = icons_image,
			icon = icons[i],
			characterid = i
		}
		basefactory.create(newcharacter)
		factory.create(newcharacter)
		characters[i] = newcharacter
	end

	for i=1, 3 do
		players[i] = {
			viewport = nil,
			character = nil,
		}
	end
end

function charactermanager.spawn_characters(map, spawnpoints)
	spawned_characters = {}

	for i, spawnpoint in ipairs(spawnpoints) do
		local character
		if spawnpoint.properties.default then
			characterid = find(default_character, spawnpoint.properties.default)
			character = characters[characterid]
		else
			character = hajtech.viewports[i].character
		end

		character.x, character.y = spawnpoint.x, spawnpoint.y
		character.layer = spawnpoint.layer
		map:removeEntity(spawnpoint)
		if character.reset then character:reset() end
		map:addEntity(character)
	end
end

function charactermanager.get_free_viewport()
	local occupied = {}
	for i, player in ipairs(players) do
		if player.viewport then
			occupied[player.viewport] = true
		end
	end
	for i=1,3 do
		if not occupied[i] then
			return i
		end
	end
end

function charactermanager.connect_player_to_viewport(player)
	local viewport = charactermanager.get_free_viewport()
	players[player].viewport = viewport
	hajtech.viewports[viewport].player = player

	character = hajtech.viewports[viewport].character
	character.player = player
	players[player].character = character
end

charactermanager.characters = characters
charactermanager.players = players

return charactermanager