
local collision = {}

local map

function collision.setMap(newmap)
	map = newmap
end

function collision.moveX(obj, distance)
	if distance == 0 then return end

	local startx = obj.x
	if distance > 0 then
		startx = startx + obj.w
	end
	local endx = startx + distance

	local ytop = obj.y
	local ybottom = obj.y + obj.h - 1

	ytop = math.floor(ytop / map.tileheight) + 1
	ybottom = math.floor(ybottom / map.tileheight) + 1

	startx = math.floor(startx / map.tilewidth) + 1
	endx = math.floor(endx / map.tilewidth) + 1

	local newx = obj.x + distance
	local collidee = nil

	local function bleh()
		for x = startx, endx, sign(distance) do
			if x < 1 or x > map.width then break end

			for y = ytop, ybottom do
				if y < 1 or y > map.height then break end

				for layerid = #map.layers, 1, -1 do
					local layer = map.layers[layerid]
					if layer.type == "tilelayer" then
						local tile = layer.properties.solid and layer.data[y][x]

						if tile and tile.properties.solid then
							collidee = tile
							newx = (x - 1) * map.tilewidth
							if distance > 0 then
								newx = newx - obj.w
							else
								newx = newx + map.tilewidth
							end
							return
						end
					end
				end
			end
		end
	end
	bleh()

	obj.w = obj.w + math.abs(distance) + 1
	if distance < 0 then obj.x = obj.x + distance - 1 end
	local collisions = collision.overlapWithEntity(obj, 'solid')
	if distance < 0 then obj.x = obj.x - distance + 1 end
	obj.w = obj.w - math.abs(distance) - 1
	for i = 1, #collisions do
		local candidate = collisions[i]
		if distance > 0 then
			local candidatex = candidate.x - obj.w
			if candidatex < newx then
				newx = candidatex
				collidee = candidate
				--print("distance: " .. distance .. " newx: " .. newx)
			end
		else
			local candidatex = candidate.x + candidate.w
			if candidatex > newx then
				newx = candidatex
				collidee = candidate
				--print("distance: " .. distance .. " newx: " .. newx)
			end
		end
	end

	obj.x = newx
	return collidee
end

function collision.moveY(obj, distance)
	if distance == 0 then return end

	local starty = obj.y-1
	if distance > 0 then
		starty = starty + obj.h+1
	end
	local endy = starty + distance

	local xleft = obj.x
	local xright = obj.x + obj.w - 1

	xleft = math.floor(xleft / map.tilewidth) + 1
	xright = math.floor(xright / map.tilewidth) + 1

	starty = math.floor(starty / map.tileheight) + 1
	endy = math.floor(endy / map.tileheight) + 1

	local newy = obj.y + distance
	local collidee = nil

	local function bleh()
		for y = starty, endy, sign(distance) do
			if y < 1 or y > map.height then break end

			for x = xleft, xright do
				if x < 1 or x > map.width then break end

				for layerid = #map.layers, 1, -1 do
					local layer = map.layers[layerid]
					if layer.type == "tilelayer" then
						local tile = layer.properties.solid and layer.data[y][x]

						local collide = false
						if tile and tile.properties.solid then
							collide = true
						end

						if tile and tile.properties.ledge then
							if (obj.yspeed or -1) > 0 and distance > 0 then
								if y > starty or (obj.y + obj.h) % map.tileheight < 2 then
									collide = true
								end
							end
						end

						if collide then
							collidee = tile
							newy = (y - 1) * map.tileheight
							if distance > 0 then
								newy = newy - obj.h
							else
								newy = newy + map.tileheight
							end
							return
						end
					end
				end
			end
		end
	end
	bleh()

	obj.h = obj.h + math.abs(distance) + 1
	if distance < 0 then obj.y = obj.y + distance - 1 end
	local collisions = collision.overlapWithEntity(obj, 'solid')
	if distance < 0 then obj.y = obj.y - distance + 1 end
	obj.h = obj.h - math.abs(distance) - 1
	for i = 1, #collisions do
		local candidate = collisions[i]
		if distance > 0 then
			local candidatey = candidate.y - obj.h
			if candidatey < newy then
				newy = candidatey
				collidee = candidate
			end
		else
			local candidatey = candidate.y + candidate.h
			if candidatey > newy then
				newy = candidatey
				collidee = candidate
			end
		end
	end

	obj.y = newy
	return collidee
end

function collision.overlapWith(obj, property)
	local xleft = obj.x
	local xright = obj.x + obj.w - 1
	xleft = math.floor(xleft / map.tilewidth) + 1
	xright = math.floor(xright / map.tilewidth) + 1

	local ytop = obj.y
	local ybottom = obj.y + obj.h - 1
	ytop = math.floor(ytop / map.tileheight) + 1
	ybottom = math.floor(ybottom / map.tileheight) + 1

	for y = ytop, ybottom do
		if y < 1 or y > map.height then break end
		for x = xleft, xright do
			if x < 1 or x > map.width then break end
			for layerid = #map.layers, 1, -1 do
				local layer = map.layers[layerid]
				if layer.type == "tilelayer" then
					local tile = layer.properties.solid and layer.data[y][x]

					if tile and tile.properties and tile.properties[property] then
						return tile, (x - 0.5) * map.tilewidth, (y - 0.5) * map.tileheight
					end
				end
			end
		end
	end

	return nil
end

function collision.overlapWithEntity(obj, property, value)
	local x1, x2 = obj.x + 1, obj.x + obj.w - 1
	local y1, y2 = obj.y + 1, obj.y + obj.h - 1
	local matches = {}

	for layerid = #map.layers, 1, -1 do
		local layer = map.layers[layerid]
		if layer.type == "objectgroup" then
			for _, entity in pairs(layer.entities) do
				if entity ~= obj and not (
					x2 <= entity.x or
					x1 >= entity.x + entity.w or
					y2 <= entity.y or
					y1 >= entity.y + entity.h
					) then
					if not property or
						(not value and entity.properties[property]) or
						(value ~= nil and entity.properties[property] == value) then
						--print("x1: " .. x1 .. " x2: " .. x2 .. " y1: " .. y1 .. " y2: " .. y2)
						matches[#matches+1] = entity
					end
				end
			end
		end
	end

	return matches
end

function collision.entitiesAtPoint(x, y, tempmap)
	local map = tempmap or map
	local matches = {}

	for layerid = #map.layers, 1, -1 do
		local layer = map.layers[layerid]
		if layer.type == "objectgroup" then
			for _, entity in pairs(layer.entities or {}) do
				if not (
					x <= entity.x or
					x >= entity.x + entity.width or
					y <= entity.y or
					y >= entity.y + entity.height
					) then
					matches[#matches+1] = entity
				end
			end
		end
	end

	return matches
end

return collision
