
local button = {}

function button.create(object)
	object.properties.usable = true
	object.use = button.use
	object.setstate = button.setstate
	object.update = button.update
	object.state = not object.properties.disabled

	if object.properties.disabled then
		object.offgid = object.gid
		object.ongid = object.gid + 1
	else
		object.ongid = object.gid
		object.offgid = object.gid - 1
	end
end

function button:update(dt)
	if not self.started then
		self.state = not self.state
		self:use()
		self.started = true
	end

	if self.rotspeed then
		local rotdiff = math.rad(45) - self.rotation
		self.rotspeed = self.rotspeed + rotdiff
		self.rotation = self.rotation + rotdiff*dt*2
		self.rotation = self.rotation + self.rotspeed*dt

		if math.abs(self.rotspeed) < math.rad(5) and math.abs(rotdiff) < math.rad(5) then
			self.rotspeed = nil
		end
	end
end

function button:use(user)
	if self.properties.broken and self.started then
		if not self.broke then
			self.broke = true

			self.rotation = 0
			self.rotspeed = 0.2
			local tile = hajtech.map.tiles[self.gid]
			self.draw = function(self)
				love.graphics.draw(tile.tileset.image,
					tile.quad,
					self.x,
					self.y,
					self.rotation)
			end
		end
		return
	end

	self:setstate(not self.state)

	for _, target in ipairs(self.outgoing_links or {}) do
		if target.setstate then
			target:setstate(self.state)
		end
	end
end

function button:setstate(state)
	self.state = state
	self.gid = self.state and self.ongid or self.offgid
end

return button
