
local bunny = {}

local texture = love.graphics.newImage('img/character-bunny.png')
local viewport_border = love.graphics.newImage('img/player3_border.png')

function bunny.create(object)
	object.properties.character = "bunny"
	object.title = "First Mate Bucket"

	object.reset = bunny.reset
	object.press_jump = bunny.press_jump
	object.release_jump = bunny.release_jump
	object.update = bunny.update
	object.viewport_border = viewport_border
	object.properties.throwable = true
	object.properties.ventilatable = true
	object.facing = 1

	-- Animations
	object.texture = texture
	local framesize = 16

	object.animations = {
		idle = hajtech.animation.new(texture, 2, 0, 1, framesize),
		walking = hajtech.animation.new(texture, 20, 8, 13, framesize),
		dying = hajtech.animation.new(texture, 6, 16, 20, framesize),
		jumping = hajtech.animation.new(texture, 0, 32, 32, framesize),
		falling = hajtech.animation.new(texture, 0, 33, 33, framesize),
	}

	object:reset()
end

function bunny:reset()
	self.w = 16
	self.h = 16
	self.xspeed = 0
	self.yspeed = 0
	self.onground = false
	self.currentanimation = self.animations.idle
	self.update = bunny.update
	self.visible = true
end

function bunny:press_jump()
	if self.onground then
		self.yspeed = -510

		if hajtech.input.getButtons(self.player).down then
			self.yspeed = 0
			hajtech.collision.moveY(self, 4)
		end
	end

	if self.carriedby then
		self.yspeed = -300
		self.carriedby.carrying = nil
		self.carriedby = nil
		hajtech.map:addEntity(self)
	end
end

function bunny:release_jump()
	if self.yspeed < -200 then
		self.yspeed = -200
	end
end

function bunny:update(dt)
	self.currentanimation:update(dt)

	local x, y = hajtech.input.getXY(self.player)
	local buttons = hajtech.input.getButtons(self.player)
	local nextanimation = "idle"

	if x > 0 then
		self.facing = 1
	elseif x < 0 then
		self.facing = -1
	end

	if self.onground then
		if self.xspeed ~= 0 then
			nextanimation = "walking"
			self.animations.walking.speed = math.abs(x)
		end
	else
		if self.yspeed > 80 then
			nextanimation = "falling"
		elseif self.yspeed < -20 then
			nextanimation = "jumping"
		else
			nextanimation = "idle"
		end
	end

	local tile = hajtech.collision.moveY(self, self.yspeed*dt)
	if tile then
		if self.yspeed > 0 then
			self.onground = true
		end

		self.yspeed = 0

		if tile.properties.spikes then
			self.xspeed = -tile.sx * 300
			self.yspeed = -200
			self.onground = false
		end
	else
		self.onground = false
	end
	if hajtech.collision.moveX(self, self.xspeed*dt) then
		self.xspeed = 0
	end

	-- xspeed
	local targetspeed = x*260
	local dspeed = targetspeed - self.xspeed
	if sign(targetspeed) ~= sign(self.xspeed) and targetspeed ~= 0 then
		-- retardaccel
		speedchange = 1200*dt
		dspeed = clamp(dspeed, speedchange)
	elseif math.abs(targetspeed) < math.abs(self.xspeed) then
		-- retard
		speedchange = 800*dt
		dspeed = clamp(dspeed, speedchange)
	else
		-- accel
		speedchange = 400*dt
		dspeed = clamp(dspeed, speedchange)
	end
	self.xspeed = self.xspeed + dspeed

	-- Apply gravity
	self.yspeed = math.min(256, self.yspeed + 900*dt)

	-- Disregard gravity
	if buttons.fly then
		self.yspeed = 128*y
	end

	self:transit(nextanimation)
end

return bunny
