-- General character

local character = {}

local viewport_border = love.graphics.newImage('img/player1_border.png')

function character.create(object)
	object.type = "character"
	object.visible = true
	object.properties = {}

	object.press_jump = character.press_jump
	object.press_use = character.press_use
	object.use = character.use
	object.draw = character.draw
	object.transit = character.transit

	object.viewport_border = viewport_border
	object.properties.usable = true

	object.player = nil
	object.viewport = nil
end

function character:press_use()
	local matches = hajtech.collision.overlapWithEntity(self, "usable")
	for _, entity in ipairs(matches) do
		if entity.use then
			entity:use(self)
		end
	end
end

function character:use(user)
	-- Switch to this character
	if self.viewport then return end
	if user.type ~= "character" then return end

	self.player = user.player
	self.viewport = user.viewport
	user.player = nil
	user.viewport = nil

	local player = hajtech.charactermanager.players[self.player]
	player.character = self
	self.viewport.character = self
end

function character:press_jump()
	if self.onground then
		self.yspeed = -120

		if hajtech.input.getButtons(self.player).down then
			self.yspeed = 0
			hajtech.collision.moveY(self, 4)
		end
	end
end

function character:draw()
	local x, y = self.x + self.w/2, self.y + self.h/2
	local sx = 1

	if self.facing < 0 then
		sx = -sx
	end

	local w, h = self.currentanimation:getDimensions()
	if self.viewport == nil then
		love.graphics.setShader(hajtech.grayscale)
	end
	love.graphics.draw(self.texture, self.currentanimation:getFrame(),
		math.floor(x), math.floor(y), rot, sx, 1, w/2, h/2)
	love.graphics.setShader(nil)
end

function character:transit(animation)
	local newanimation = self.animations[animation]

	if self.currentanimation ~= newanimation then
		newanimation:reset()
		self.currentanimation = newanimation
	end
end

return character
