
local mermaid = {}

local texture = love.graphics.newImage('img/character-mermaid.png')

function mermaid.create(object)
	object.properties.character = "mermaid"
	object.title = "Siren Pearl"

	object.reset = mermaid.reset
	object.update = mermaid.update

	object.facing = 1

	-- Animations
	object.texture = texture
	local framesize = 64

	object.animations = {
		idle = hajtech.animation.new(texture, 0, 0, 0, framesize)
	}

	object:reset()
end

function mermaid:reset()
	self.w = 30
	self.h = 56
	self.xspeed = 0
	self.yspeed = 0
	self.onground = false
	self.currentanimation = self.animations.idle
end

function mermaid:update(dt)
	self.currentanimation:update(dt)
	local x, y = hajtech.input.getXY(self.player)

	if x > 0 then
		self.facing = 1
	elseif x < 0 then
		self.facing = -1
	end

	-- xspeed
	local targetspeed = x*160
	local dspeed = targetspeed - self.xspeed
	if sign(targetspeed) ~= sign(self.xspeed) and targetspeed ~= 0 then
		-- retardaccel
		speedchange = 1200*dt
		dspeed = clamp(dspeed, speedchange)
	elseif math.abs(targetspeed) < math.abs(self.xspeed) then
		-- retard
		speedchange = 800*dt
		dspeed = clamp(dspeed, speedchange)
	else
		-- accel
		speedchange = 400*dt
		dspeed = clamp(dspeed, speedchange)
	end
	self.xspeed = self.xspeed + dspeed

	if hajtech.collision.moveX(self, self.xspeed*dt) then
		self.xspeed = 0
	end

	-- yspeed
	self.yspeed = math.min(256, self.yspeed + 600*dt)
	if hajtech.collision.moveY(self, self.yspeed*dt) then
		if self.yspeed > 0 then
			self.onground = true
		end

		self.yspeed = 0
	else
		self.onground = false
	end
end

return mermaid
