
local rope = {}

function rope.new(wielder)
	local object = {
		w = 5,
		h = 5,
		yspeed = -300,
		xspeed = 300,
		direction = 1,
		state = false,
		wielder = wielder,
		visible = true,

		draw = rope.draw,
		update = rope.update,
		launchrope = rope.launchrope,
		droprope = rope.droprope,
	}

	return object
end

function rope:draw()
	if not self.state then return end

	local colorr, colorg, colorb = love.graphics.getColor()
	love.graphics.setColor(0, 0, 0)
	love.graphics.line(self.startx, self.starty, self.x, self.y)
	love.graphics.setColor(colorr, colorg, colorb)
	love.graphics.draw(self.texture, self.quad, self.x, self.y,
		self.angle - math.rad(135), 1, 1, 5, 5)
end

function rope:update(dt)
	local wielder = self.wielder
	local x, y = hajtech.input.getXY(self.wielder.player)

	if self.state == 'launch' then
		if wielder.onground then
			wielder.xspeed = 0
			self.startx, self.starty = wielder.x + wielder.w / 2, wielder.y + 33
		else
			wielder.xspeed = self.launchxspeed
			self.startx, self.starty = wielder.x + wielder.w/2, wielder.y + wielder.h/2
		end

		ropecollision = hajtech.collision.moveX(self, self.xspeed*dt * self.direction)
		ropecollision = ropecollision or hajtech.collision.moveY(self, self.yspeed*dt)

		local dx = self.startx - self.x
		local dy = self.starty - self.y
		self.length = math.sqrt(dx*dx + dy*dy)

		if self.length > 32*5 then
			self:droprope()
		elseif ropecollision then
			if not ropecollision.properties.nohook then
				self.state = 'hang'
				--hajtech.collision.moveX(wielder, 16 * wielder.facing)
				--hajtech.collision.moveY(wielder, -16)
				self.startx, self.starty = wielder.x + wielder.w/2, wielder.y + wielder.h/2

				local dx = self.startx - self.x
				local dy = self.starty - self.y
				self.length = math.sqrt(dx*dx + dy*dy)
				self.length = self.length*0.8
				self.angle = math.atan2(dy, dx)
				self.rotationspeed = -10 / (self.length / 32) * self.direction
			else
				self:droprope()
			end
		end
	elseif self.state == 'hang' then
		self.wielder.xspeed = 0
		self.wielder.yspeed = 0

		local dx = self.startx - self.x
		local dy = self.starty - self.y
		--self.length = math.sqrt(dx*dx + dy*dy)
		self.angle = math.atan2(dy, dx)
		self.rotationspeed = -10 / (self.length / 32) * self.direction * math.sin(self.angle)
		if math.abs(self.rotationspeed) < 1 then
			self.direction = self.angle > math.rad(90) and 1 or -1
		end


		--self.length = self.length + y*128*dt
		if self.length > 32 * 1.5 then
			self.length = self.length + -100*dt
		end
		--print("rope length: " .. self.length .. " y: " .. y)

		local ox, oy = wielder.x, wielder.y
		local oangle = self.angle

		self.angle = self.angle + self.rotationspeed*dt
		local dx = (self.x + math.cos(self.angle) * self.length - wielder.w / 2) - wielder.x
		local dy = (self.y + math.sin(self.angle) * self.length - wielder.h / 2) - wielder.y
		wielder.h = wielder.h-15
		wielder.y = wielder.y+15
		if hajtech.collision.moveX(wielder, dx) or hajtech.collision.moveY(wielder, dy) then
			wielder.x, wielder.y = ox, oy+15
			self.angle = oangle
			self.direction = -self.direction
		end
		wielder.h = wielder.h+15
		wielder.y = wielder.y-15
		self.startx, self.starty = wielder.x + wielder.w/2, wielder.y + wielder.h/2

		--print("length: " .. self.length)
		--print("angle: " .. math.deg(self.angle))
		if math.abs(math.deg(self.angle) - 90) > 55 then
			--self:droprope()
		end
		if self.length < 10 then
			self:droprope()
		end
	else
		return false
	end

	return true
end

function rope:launchrope()
	if not self.state then
		self.state = 'launch'
		self.x, self.y = self.wielder.x + self.wielder.w / 2, self.wielder.y + 33
		self.startx, self.starty = self.x, self.y
		self.direction = self.wielder.facing
		self.launchxspeed = self.wielder.xspeed
		self.layer = self.wielder.layer
		hajtech.map:addEntity(self)

		self.angle = (self.wielder.facing > 0) and math.rad(135) or math.rad(45)
	end
end

function rope:droprope()
	if self.state then
		if self.state == 'hang' then
			self.wielder.xspeed = 200 * self.direction
			self.wielder.yspeed = -50
		end
		self.state = false
		hajtech.map:removeEntity(self)
	end
end

return rope
