
local door = {}

local legacytextures = {
	love.graphics.newImage("img/door1.png"),
	love.graphics.newImage("img/door2.png"),
	love.graphics.newImage("img/door3.png"),
	love.graphics.newImage("img/door1_horizontal.png"),
	love.graphics.newImage("img/door2_horizontal.png"),
	love.graphics.newImage("img/door4_horizontal.png")
}

function door.create(object)
	object.setstate = door.setstate
	object.update = door.update
	object.draw = door.draw

	if object.properties.image then
		object.image = love.graphics.newImage(object.properties.image)
	else
		if object.properties.horizontal then
			if object.width == 32 then
				object.image = legacytextures[4]
			elseif object.width == 64 then
				object.image = legacytextures[5]
			elseif object.width == 128 then
				object.image = legacytextures[6]
			else
				error("Door of wrong size")
			end
		else
			if object.height == 32 then
				object.image = legacytextures[1]
			elseif object.height == 64 then
				object.image = legacytextures[2]
			elseif object.height == 96 then
				object.image = legacytextures[3]
			else
				error("Door of wrong size")
			end
		end
	end
	object.image:setFilter("nearest", "nearest")

	local offsetY = (object.properties.skin or 0) * object.h
	local iw, ih = object.image:getDimensions()
	object.animation = hajtech.animation.new(object.image,
		10,
		0, iw/object.w - 1,
		object.w, object.h,
		0, offsetY)
	object.animation.wrap = false
	object.animation.speed = -1
end

function door:setstate(state)
	for _, target in ipairs(self.outgoing_links or {self}) do
		-- door closing
		if not target.properties.solid and state == false and not self.properties.horizontal then
			local matches = hajtech.collision.overlapWithEntity(target, "character")
			for i, match in ipairs(matches) do
				local diff = (match.x + match.w/2) - (target.x + target.w/2)
				local dist
				if diff > 0 then
					dist = match.x - (target.x + target.w)
				else
					dist = (match.x + match.w) - target.x
				end
				hajtech.collision.moveX(match, -dist)
			end
		end

		target.properties.solid = not state
	end
	self.animation.speed = state and 1 or -1

	-- sync other buttons controlling this entity
	for _, source in ipairs(self.incoming_links or {}) do
		if source.type == "button" and source.setstate then
			source:setstate(state)
		end
	end
end

function door:update(dt)
	if not self.initialized then
		self.initialized = true

		local course = self.properties.course
		self:setstate(course and hajtech.persistent.state.courserecords[course])
	end

	self.animation:update(dt)
end

function door:draw()
	love.graphics.draw(self.image, self.animation:getFrame(), self.x, self.y)
end

return door
