
local levelexit = {}

local texture = love.graphics.newImage("img/levelexit.png")

function levelexit.create(object)
	object.draw = levelexit.draw
	object.update = levelexit.update
	object.getcourse = levelexit.getcourse
	object.currentmatched = 0

	local w, h = texture:getDimensions()
	if object.properties.mode == "start" then
		object.quad = love.graphics.newQuad(0, 0, 64, 64, w, h)
	elseif object.properties.mode == "end" then
		object.quad = love.graphics.newQuad(0, 64, 64, 64, w, h)
		object.name = object.name .. "_end"
	elseif object.properties.mode == "previous" then
		object.quad = love.graphics.newQuad(0, 128, 64, 64, w, h)
		object.name = "__startofmap"
	elseif object.properties.mode == "next" then
		object.quad = love.graphics.newQuad(0, 192, 64, 64, w, h)
		object.name = "__endofmap"
	end
end

function levelexit:getcourse()
	if self.properties.mode == "end" then
		return string.sub(self.name, 1, #self.name - 4)
	else
		return self.name
	end
end

function levelexit:draw()
	if self.quad then
		love.graphics.draw(texture, self.quad, self.x, self.y)
	end
	if self.currentmatched > 0 then
		love.graphics.printf(self.currentmatched .. "/3", self.x, self.y, self.w, 'center')
	end
end

function levelexit:update(dt)
	if not self.started then
		self.started = true
		for _, entity in ipairs(self.outgoing_links or {}) do
			if entity.type == "sign" then
				if self.properties.mode == "start" then
					entity.properties.text = string.upper(self:getcourse())
				elseif self.properties.mode == "end" then
					local newtext = string.upper(self:getcourse())
					local record = hajtech.persistent.formattime(
						hajtech.persistent.state.courserecords[self:getcourse()])
					if record then
						newtext = newtext .. "\n" .. record
					end
					entity.properties.text = newtext
				elseif self.properties.mode == "previous" then
					entity.properties.text = "BACK"
				elseif self.properties.mode == "next" then
					entity.properties.text = "NEXT"
				end
				break
			end
		end
	end

	self.currentmatched = 0
	local matches = hajtech.collision.overlapWithEntity(self, 'character')
	for i, match in ipairs(matches) do
		self.currentmatched = self.currentmatched + 1
	end

	if self.properties.mode == "end" then
		if not hajtech.persistent.state.courserecords[self:getcourse()] then
			self.currentmatched = 0
		end
	end

	if self.currentmatched >= 3 then
		if self.properties.mode == "next" then
			hajtech.persistent.nextMap()
		elseif self.properties.mode == "previous" then
			hajtech.persistent.previousMap()
		elseif self.properties.mode == "start" then
			hajtech.persistent.enterCourse(self:getcourse())
		elseif self.properties.mode == "end" then
			hajtech.persistent.enterCourseEnd(self:getcourse())
		elseif self.properties.nextlevel then
			hajtech.change_level(self.properties.nextlevel, self.properties.spawnprefix)
			if self.properties.cinematic then
				hajtech.cinematic.play(self.properties.cinematic)
			end
		end
	end
end

return levelexit
