
local sprite = {}

function sprite.spawneffect(x, y, layer, image, lifespan, animspeed)
	local iw, ih = image:getDimensions()
	local animation = hajtech.animation.new(image,
		animspeed or 1,
		0, iw/ih - 1,
		ih, ih)

	local effect = sprite.new(x, y, image, animation, lifespan)
	effect.layer = layer
	hajtech.map:addEntity(effect)
end

function sprite.new(x, y, image, animation, lifespan)
	local object = {
		xspeed = 0, yspeed = 0,
		visible = true,
		image = image,
		animation = animation,

		reset = sprite.reset,
		draw = sprite.draw,
		update = sprite.update,

		uptime = 0,
		lifespan = lifespan or 0,
	}

	-- center
	object.w, object.h = animation:getDimensions()
	object.x, object.y = x - object.w/2, y - object.h/2

	-- lifespan < 0 means animation plays once
	if object.lifespan < 0 then
		object.animation.wrap = false
	end

	return object
end

function sprite:reset(x, y)
	self.x, self.y = x - self.w/2, y - self.h/2
	self.uptime = 0
	self.animation:reset()
end

function sprite:update(dt)
	self.uptime = self.uptime + dt
	if self.lifespan > 0 and self.uptime >= self.lifespan then
		if self.layer then
			hajtech.map:removeEntity(self)
		end
		return
	end

	self.animation:update(dt)
	if self.lifespan < 0 and self.animation:done() then
		if self.layer then
			hajtech.map:removeEntity(self)
		end
		return
	end

	self.x = self.x + self.xspeed*dt
	self.y = self.y + self.yspeed*dt
end

function sprite:draw()
	love.graphics.draw(
			self.image,
			self.animation:getFrame(),
			self.x,
			self.y)
end

return sprite
