
local supply = {}   -- Barrel
local supplee = {}  -- Fruit

local texture = love.graphics.newImage('img/throwable-items.png')
local framesize = 32

local barreltexture = love.graphics.newImage('img/barrel.png')

function supply.create(object)
	-- object.properties.usable = true
	object.properties.supply = true
	object.use = supply.use
	object.draw = supply.draw

	local offsetY = (object.properties.skin or 0) * object.h
	object.currentanimation = hajtech.animation.new(barreltexture, 0,
		0, 0, object.w, object.h, 0, offsetY)
end

function supply:use(user)
	if user.properties.character ~= "sailor" or user.carrying then return end
	local skin = self.properties.skin or 0

	local t = {}
	t.w = 4
	t.h = 4
	t.x = 0
	t.y = 0
	t.xspeed = 0
	t.yspeed = 0
	t.rot = 0
	t.update = supplee.update
	t.draw = supplee.draw
	t.layer = user.layer
	t.visible = true
	t.usedentities = {}
	t.properties = {}
	t.properties.ventilatable = true
	t.animations = {
		fly = hajtech.animation.new(texture, 0, 0 + 8*skin, 0 + 8*skin, framesize),
		splat = hajtech.animation.new(texture, 10, 0 + 8*skin, 5 + 8*skin, framesize),
	}
	t.animations.splat.wrap = false
	t.animation = t.animations.fly

	user.carrying = t
end

function supply:draw()
	love.graphics.draw(barreltexture, self.currentanimation:getFrame(), self.x, self.y)
end

function supplee:update(dt)
	local matches = hajtech.collision.overlapWithEntity(self, "usable")
	for _, entity in ipairs(matches) do
		if entity.use then
			if not self.usedentities[entity] then
				entity:use(self)
				self.usedentities[entity] = true
				if entity.type == "ventilation" then
					local target = entity:gettarget()
					if target then self.usedentities[target] = true end
				end
			end
		end
	end

	if hajtech.collision.moveY(self, self.yspeed*dt) then
		self.rot = math.rad(90)
		if self.yspeed < 0 then
			self.rot = math.rad(-90)
		end
		self.animation = self.animations.splat
		self.update = supplee.splat
		return
	end
	if hajtech.collision.moveX(self, self.xspeed*dt) then
		if self.xspeed < 0 then
			self.rot = math.rad(180)
		else
			self.rot = math.rad(0)
		end
		self.animation = self.animations.splat
		self.update = supplee.splat
		return
	end

	local dspeed = 0 - self.xspeed
	local speedchange = 20*dt
	dspeed = clamp(dspeed, speedchange)
	self.xspeed = self.xspeed + dspeed

	-- Apply gravity
	self.yspeed = math.min(256, self.yspeed + 900*dt)

	self.rot = self.rot + 20*dt
end

function supplee:splat(dt)
	-- last living moments of fame-update function
	self.animation:update(dt)
	if self.animation:done() then
		if self.viewport.entity == self then
			self.viewport.entity = nil
		end
		hajtech.map:removeEntity(self)
	end
end

function supplee:draw()
	local x, y = self.x + self.w/2, self.y + self.h/2

	local w, h = self.animation:getDimensions()
	love.graphics.draw(texture, self.animation:getFrame(),
		math.floor(x), math.floor(y), self.rot, 1, 1, w/2, h/2)
end

return supply
