
       local input = {}

input.playerbindings = {}

input.keyboardtemplates = {
	{
		action2key = {
			up = 'w',
			left = 'a',
			down = 's',
			right = 'd',
			use = 'q',
			jump = 'z',
			action1 = 'x',
			--action2 = 'c',
			fly = 'c',
			swap = 'e',
		},
		key2action = {}
	},
	{
		action2key = {
			up = 'up',
			left = 'left',
			down = 'down',
			right = 'right',
			use = 'rshift',
			jump = ',',
			action1 = '.',
			--action2 = '/',
			fly = '/',
			swap = '\'',
		},
		key2action = {}
	},
	{
		action2key = {
			up = 'i',
			left = 'j',
			down = 'k',
			right = 'l',
			use = 'u',
			jump = 'b',
			action1 = 'n',
			--action2 = 'm',
			fly = 'm',
			swap = 'o',
		},
		key2action = {}
	},
}

input.gamepadtemplates = {
	{
		x = 'leftx',
		y = 'lefty',
		action2key = {
			up = 'dpup',
			left = 'dpleft',
			down = 'dpdown',
			right = 'dpright',
			use = 'b',
			jump = 'a',
			action1 = 'x',
			action2 = 'y',
			--fly = 'y',
			swap = 'leftshoulder',
		},
		key2action = {}
	},
}

local keypressed = {}
local gamepadpressed = {}

function input.loadBindings()
	-- todo: load bindings from file

	for _, template in ipairs(input.keyboardtemplates) do
		for action, key in pairs(template.action2key) do
			template.key2action[key] = action
		end
	end

	for _, template in ipairs(input.gamepadtemplates) do
		for action, key in pairs(template.action2key) do
			template.key2action[key] = action
		end
	end
end

function input.getkeymapping(key, scancode)
	local bind = keypressed[scancode]

	if bind then
		return bind[1], bind[2]
	end

	if #input.playerbindings >= 3 then
		return
	end

	local templateid = containingTemplate(scancode, input.keyboardtemplates)
	if templateid then
		local playerid = #input.playerbindings + 1
		local template = input.keyboardtemplates[templateid]
		input.playerbindings[playerid] = template
		hajtech.charactermanager.connect_player_to_viewport(playerid)

		table.remove(input.keyboardtemplates, templateid)

		for scancode, action in pairs(template.key2action) do
			keypressed[scancode] = {playerid, action}
		end

		bind = keypressed[scancode]
		return bind[1], bind[2]
	end
end

function containingTemplate(pressed, templates)
	for id, template in ipairs(templates) do
		if template.key2action[pressed] then
			return id
		end
	end
end


function input.gamepadpressed(joystick, button)
	local bindmap = gamepadpressed[joystick]

	if bindmap then
		local bind = bindmap[button]
		if bind then
			return bind[1], bind[2]
		end
	end

	if #input.playerbindings >= 3 then
		return
	end

	local templateid = containingTemplate(button, input.gamepadtemplates)
	if templateid then
		local playerid = #input.playerbindings + 1
		local template = input.gamepadtemplates[templateid]
		input.playerbindings[playerid] = {
			gamepad = joystick,
			x = template.x,
			y = template.y,
			action2key = template.action2key,
			key2action = template.key2action,
		}
		hajtech.charactermanager.connect_player_to_viewport(playerid)

		gamepadpressed[joystick] = gamepadpressed[joystick] or {}
		for button, action in pairs(template.key2action) do
			gamepadpressed[joystick][button] = {playerid, action}
		end

		bindmap = gamepadpressed[joystick]
		if bindmap then
			local bind = bindmap[button]
			if bind then
				return bind[1], bind[2]
			end
		end
	end
end

function input.getButtons(playerid)
	local playerbinds = input.playerbindings[playerid]
	local buttons = {}

	if playerbinds then
		local keys = playerbinds.action2key
		if playerbinds.gamepad then
			local gamepad = playerbinds.gamepad

			local x, y = input.getXY(playerid)
			if x < 0 then buttons.left = true end
			if x > 0 then buttons.right = true end
			if y < 0 then buttons.up = true end
			if y > 0 then buttons.down = true end

			for action, button in pairs(keys) do
				buttons[action] = buttons[action] or gamepad:isGamepadDown(button)
			end
		else
			for action, scancode in pairs(keys) do
				buttons[action] = love.keyboard.isScancodeDown(scancode)
			end
		end
	end

	return buttons
end

function input.getXY(playerid)
	local playerbinds = input.playerbindings[playerid]
	local x, y = 0, 0

	if playerbinds then
		local keys = playerbinds.action2key
		if playerbinds.gamepad then
			local gamepad = playerbinds.gamepad
			if playerbinds.x then x = gamepad:getGamepadAxis(playerbinds.x) end
			local deadzone = 0.2
			if x < deadzone and x > -deadzone then x = 0 end
			deadzone = 0.4
			if playerbinds.y then y = gamepad:getGamepadAxis(playerbinds.y) end
			if y < deadzone and y > -deadzone then y = 0 end
			if keys.left and gamepad:isGamepadDown(keys.left) then x = -1 end
			if keys.right and gamepad:isGamepadDown(keys.right) then x = 1 end
			if keys.up and gamepad:isGamepadDown(keys.up) then y = -1 end
			if keys.down and gamepad:isGamepadDown(keys.down) then y = 1 end
		else
			local isScancodeDown = love.keyboard.isScancodeDown
			if keys.left and isScancodeDown(keys.left) then x = x - 1 end
			if keys.right and isScancodeDown(keys.right) then x = x + 1 end
			if keys.up and isScancodeDown(keys.up) then y = -1 end
			if keys.down and isScancodeDown(keys.down) then y = 1 end
		end
	end

	return x, y
end

return input
