
local persistent = {}

persistent.hubworld = "stage_select"

local state
local timer
local timerpaused
local currentcourse
local currentmap
function persistent.resetGame()
	state = {
		courserecords = {},
		maprecords = {},
		mapflasks = {},
	}
	persistent.state = state
	timer = 0
	timerpaused = false
	currentcourse = nil
	currentmap = 0
	totalflasks = 0
end
persistent.resetGame()

local function getmap(course, map)
	local mapname = course .. "_" .. map
	print("looking for " .. mapname)
	local result = pcall(require, "maps." .. mapname)
	print("result: " .. (result and "yeah" or "no"))
	return result and mapname
end

function persistent.enterCourse(course)
	state.maprecords[course] = state.maprecords[course] or {}

	local nextmap = getmap(course, 1)
	if nextmap then
		currentcourse = course
		currentmap = 1
		hajtech.transition.fade(0.3, 0.2,
			hajtech.change_level, nextmap, "__startofmap")
		timer = -1
		timerpaused = false
	else
		error("Couldn't find start of course: " .. course)
	end
end

function persistent.resumeCourse(course)
	state.maprecords[course] = state.maprecords[course] or {}

	local lastmap = #state.maprecords[course]
	local nextmap = getmap(course, lastmap+1)
	if nextmap then
		currentcourse = course
		currentmap = lastmap+1
		hajtech.transition.fade(0.3, 0.2,
			hajtech.change_level, nextmap, "__startofmap")
		timer = -1
		timerpaused = false
	else
		error("Couldn't find level " .. lastmap+1 .. " in course: " .. course)
	end
end

function persistent.enterCourseEnd(course)
	--state.maprecords[course] = state.maprecords[course] or {}

	local lastmap = #state.maprecords[course]
	local nextmap = getmap(course, lastmap)
	if nextmap then
		currentcourse = course
		currentmap = lastmap
		hajtech.transition.fade(0.3, 0.2,
			hajtech.change_level, nextmap, "__endofmap")
		timer = -1
		timerpaused = true
	else
		error("Couldn't find end of course: " .. course)
	end
end

function persistent.exitCourse()
	print("exitCourse")
	hajtech.transition.fade(0.3, 0.5,
		hajtech.change_level, persistent.hubworld, currentcourse)
	currentcourse = nil
end

function persistent.completeCourse()
	print("completeCourse")
	local coursetime = 0
	for _, maptime in ipairs(state.maprecords[currentcourse]) do
		coursetime = coursetime + maptime
	end
	print("coursetime: " .. coursetime)
	local records = state.courserecords
	if not records[currentcourse] or records[currentcourse] > coursetime then
		records[currentcourse] = coursetime
		print("new course record!")
	end

	hajtech.transition.fade(0.3, 0.5,
		hajtech.change_level, persistent.hubworld, currentcourse .. "_end")
	currentcourse = nil
end

function persistent.nextMap()
	print("nextMap")
	if not currentcourse then
		hajtech.titlemenu.entermenu()
		return
	end

	local courserecords = state.maprecords[currentcourse]
	if timer > 0 then
		if not courserecords[currentmap] or courserecords[currentmap] > timer then
			courserecords[currentmap] = timer
			print("new record!")
		end
	end
	currentmap = currentmap + 1

	local nextmap = getmap(currentcourse, currentmap)
	if nextmap then
		hajtech.transition.fade(0.3, 0.2,
			hajtech.change_level, nextmap, "__startofmap")
		timer = -1
		timerpaused = false
	else
		persistent.completeCourse(currentcourse)
	end
end

function persistent.previousMap()
	print("previousMap")
	if not currentcourse then
		hajtech.titlemenu.entermenu()
		return
	end

	currentmap = currentmap - 1
	if currentmap < 1 then
		persistent.exitCourse()
		return
	end

	local nextmap = getmap(currentcourse, currentmap)
	if nextmap then
		hajtech.transition.fade(0.3, 0.2,
			hajtech.change_level, nextmap, "__endofmap")
		timer = -1
		timerpaused = true
	else
		error("Previous map has disappeared")
	end
end

function persistent.findFlask()
	local currentcourse = currentcourse
	if not currentcourse then
		currentcourse = persistent.hubworld
		currentmap = 1
	end

	state.mapflasks[currentcourse] = state.mapflasks[currentcourse] or {}
	state.mapflasks[currentcourse][currentmap] = (state.mapflasks[currentcourse][currentmap] or 0) + 1
	totalflasks = totalflasks + 1
end

function persistent.isCurrentFlaskTaken()
	local currentcourse = currentcourse
	if not currentcourse then
		currentcourse = persistent.hubworld
		currentmap = 1
	end

	state.mapflasks[currentcourse] = state.mapflasks[currentcourse] or {}
	local flasks = state.mapflasks[currentcourse][currentmap]
	return (flasks or 0) > 0
end

function persistent.update(dt)
	if not timerpaused then
		if timer < 0 then
			timer = 0
		else
			timer = timer + dt
		end
	end
end

function persistent.pauseTimer(pause)
	timerpaused = pause
end

function persistent.formattime(time)
	if not time or time < 0 then
		return nil
	end

	local minutes = math.floor(time / 60)
	time = time - minutes*60
	local seconds = math.floor(time)
	time = time - seconds
	local hundreths = math.floor(time*100)

	seconds = tostring(seconds)
	if #seconds < 2 then
		seconds = "0" .. seconds
	end
	hundreths = tostring(hundreths)
	if #hundreths < 2 then
		hundreths = "0" .. hundreths
	end

	return minutes .. ":" .. seconds .. "." .. hundreths
end

function persistent.getCurrentTime()
	return currentcourse and persistent.formattime(timer)
end

function persistent.getCurrentRecord()
	return currentcourse and persistent.formattime(state.maprecords[currentcourse][currentmap])
end

function persistent.getCurrentLevelname()
	return currentcourse and (string.upper(currentcourse) .. ": " .. currentmap)
end

function persistent.getTotalFlasks()
	return totalflasks
end

return persistent
