
local titlemenu = {}

local oldfunctions = {}
local currentmenu
local menustack
local menubackground = love.graphics.newImage("img/menu.png")

local function loadmenu()
	mainmenu = {
		{"New Game", titlemenu.startgame},
		{"Select Level", titlemenu.selectlevel},
		{"Play Intro", titlemenu.playintro},
		{"Exit Game", titlemenu.exitgame},
		selection = 1,
	}

	ingamemenu = {
		{"Resume", titlemenu.exitmenu},
		{"Back to hub", titlemenu.exitcourse},
		{"Quit to main menu", titlemenu.exitsubmenu},
		{"", {
			{"Take me out of here!", titlemenu.exitsubmenu},
			{"nextMap", titlemenu.call, hajtech.persistent.nextMap},
			{"previousMap", titlemenu.call, hajtech.persistent.previousMap},
			{"completeCourse", titlemenu.call, hajtech.persistent.completeCourse},
			{"exitCourse", titlemenu.call, hajtech.persistent.exitCourse},
			{"resetGame", titlemenu.call, hajtech.persistent.resetGame},
		}},
		selection = 1,
	}

	currentmenu = mainmenu
	menustack = {}
end

function titlemenu.entermenu()
	oldfunctions.draw = love.draw
	oldfunctions.update = love.update
	oldfunctions.keypressed = love.keypressed
	oldfunctions.touchmoved = love.touchmoved
	oldfunctions.handleInput = handleInput

	love.draw = titlemenu.draw
	love.update = nil
	love.keypressed = titlemenu.keypressed
	love.touchmoved = nil
	handleInput = titlemenu.handleInput

	if not currentmenu then loadmenu() end
end

function titlemenu.exitmenu()
	love.draw = oldfunctions.draw
	love.update = oldfunctions.update
	love.keypressed = oldfunctions.keypressed
	love.touchmoved = oldfunctions.touchmoved
	handleInput = oldfunctions.handleInput
end

function titlemenu.entersubmenu(menu)
	menustack[#menustack+1] = currentmenu
	currentmenu = menu
	currentmenu.selection = 1
end

function titlemenu.exitsubmenu()
	if #menustack > 0 then
		currentmenu = menustack[#menustack]
		menustack[#menustack] = nil
	end
end

function titlemenu.draw()
	hajtech.rendering.drawbegin()
	local width = hajtech.rendering.width

	love.graphics.draw(menubackground, 0, 0)

	love.graphics.setColor(love.math.colorFromBytes(20, 18, 17))
	for i = 1, #currentmenu do
		love.graphics.printf(currentmenu[i][1], 0, 270 + i*28, width, 'center')
	end

	love.graphics.setColor(love.math.colorFromBytes(149, 39, 54))
	love.graphics.circle('fill', width/3 + 60, 273 + currentmenu.selection*28 + 6, 6, 16)
	love.graphics.setColor(1, 1, 1)

	hajtech.rendering.drawend()
end

function titlemenu.keypressed(key, scancode)
	if key == 'escape' then
		if #menustack > 0 then
			titlemenu.exitsubmenu()
		else
			titlemenu.exitgame()
		end
		return
	end

	if key == 'return' then
		if not (love.keyboard.isDown('lalt') or love.keyboard.isDown('ralt')) then
			titlemenu.press_jump()
			return
		end
	end

	if tonumber(key) then
		local number = key - "0"
		if number >= 1 and number <= #currentmenu then
			currentmenu.selection = number
			titlemenu.press_jump()
			return
		end
	end

	oldfunctions.keypressed(key, scancode)
end

function titlemenu.handleInput(playerid, action, prefix)
	if titlemenu[prefix .. action] then
		titlemenu[prefix .. action]()
	end
end

function titlemenu.press_jump()
	local action = currentmenu[currentmenu.selection][2]

	if type(action) == "function" then
		local arg = {}
		for i = 3, #currentmenu[currentmenu.selection] do
			arg[#arg+1] = currentmenu[currentmenu.selection][i]
		end
		action(unpack(arg))
	elseif type(action) == "table" then
		titlemenu.entersubmenu(action)
	end
end

function titlemenu.press_up()
	currentmenu.selection = (currentmenu.selection - 1)
	if currentmenu.selection < 1 then
		currentmenu.selection = #currentmenu
	end
end

function titlemenu.press_down()
	currentmenu.selection = (currentmenu.selection + 1)
	if currentmenu.selection > #currentmenu then
		currentmenu.selection = 1
	end
end

function titlemenu.press_action1()
	titlemenu.exitsubmenu()
end

-- Menu actions --------------------------

function titlemenu.startgame()
	hajtech.persistent.resetGame()
	titlemenu.entersubmenu(ingamemenu)
	titlemenu.exitmenu()
	hajtech.transition.fade(0, 3,
		hajtech.change_level, "submarine_1")
end

function titlemenu.exitcourse()
	titlemenu.exitmenu()
	hajtech.persistent.exitCourse()
end

function titlemenu.selectlevel()
	local function enterlevel(levelname)
		titlemenu.exitmenu()
		hajtech.change_level(levelname)
	end

	menu = {}
	files = love.filesystem.getDirectoryItems("maps")
	for _, name in pairs(files) do
		name = string.match(name, "^([^_][^%.]+)%.lua$")
		if name then
			menu[#menu+1] = {name, function() enterlevel(name) end}
		end
	end

	menu[#menu+1] = {"Back", titlemenu.exitsubmenu}
	titlemenu.entersubmenu(menu)
end

function titlemenu.playintro()
	hajtech.cinematic.play('opening')
end

function titlemenu.exitgame()
	love.event.push('quit')
end

function titlemenu.call(func, ...)
	titlemenu.exitmenu()
	func(...)
end

return titlemenu
