
local transition = {}

local function savecallbacks(callbacktable)
	callbacktable.draw = love.draw
	callbacktable.update = love.update
	callbacktable.keypressed = love.keypressed
	callbacktable.touchmoved = love.touchmoved
	callbacktable.handleInput = handleInput
end

local function restorecallbacks(callbacktable)
	love.draw = callbacktable.draw
	love.update = callbacktable.update
	love.keypressed = callbacktable.keypressed
	love.touchmoved = callbacktable.touchmoved
	handleInput = callbacktable.handleInput
end

function transition.fade(timeout, timein, switchfunc, ...)
	timeout = timeout or 0.5
	timein = timein or 1
	local arg = {...}

	local oldfunctions = {}
	savecallbacks(oldfunctions)

	local fadetimer = 0

	love.draw = function()
		oldfunctions.draw()
		local w, h = love.graphics.getDimensions()
		local alpha = 1 - (timeout - fadetimer) / timeout
		love.graphics.setColor(0, 0, 0, alpha)
		love.graphics.rectangle('fill', 0, 0, w, h)
		love.graphics.setColor(1, 1, 1)
	end

	love.update = function(dt)
		fadetimer = fadetimer + dt
		if fadetimer > timeout then
			fadetimer = 0

			restorecallbacks(oldfunctions)
			switchfunc(unpack(arg))
			savecallbacks(oldfunctions)

			oldfunctions.update(0)

			love.draw = function()
				oldfunctions.draw()
				local w, h = love.graphics.getDimensions()
				local alpha = (timein - fadetimer) / timein
				love.graphics.setColor(0, 0, 0, alpha)
				love.graphics.rectangle('fill', 0, 0, w, h)
				love.graphics.setColor(1, 1, 1)
			end

			love.update = function(dt)
				oldfunctions.update(dt)
				fadetimer = fadetimer + dt
				if fadetimer > timein then
					restorecallbacks(oldfunctions)
				end
			end

			love.keypressed = nil
			love.touchmoved = nil
			handleInput = function() end
		end
	end

	love.keypressed = nil
	love.touchmoved = nil
	handleInput = function() end
end

return transition
