
local viewport = {}

function viewport.newViewport(width, height, character)
	local view = {
		x = 0,
		y = 0,
		width = width,
		height = height,
	}

	view.canvas = love.graphics.newCanvas(view.width, view.height)
	view.canvas:setFilter("nearest", "nearest")

	view.draw = viewport.draw
	view.drawcanvas = viewport.drawcanvas
	view.centerOn = viewport.centerOn
	view.follow = viewport.follow  -- if following something else than a character
	view.character = character
	view.character.viewport = view
	view.player = nil

	return view
end

function viewport.draw(view, map, x, y)
	local character = view.character

	if not view.player then
		love.graphics.setShader(hajtech.grayscale)
	end
	love.graphics.draw(character.viewport_border, x, y)
	love.graphics.setShader(nil)

	view:drawcanvas(map)
	love.graphics.draw(view.canvas, x, y+28)

	love.graphics.draw(character.icons_image, character.icon, x+4, y+2)

	love.graphics.setColor(0, 0, 0)
	love.graphics.print(character.title, x+35, y+6)
	love.graphics.setColor(1, 1, 1)
end

function viewport.drawcanvas(view, map)
	local oldcanvas = love.graphics.getCanvas()
	love.graphics.setCanvas(view.canvas)
	love.graphics.clear(map.backgroundcolor or {love.math.colorFromBytes(12, 18, 17)})

	local x, y = math.floor(view.x), math.floor(view.y)

	love.graphics.push()
	love.graphics.origin()
	love.graphics.translate(-x, -y)

	local drawrange = {
		sx = math.floor(x / map.tilewidth) + 1,
		sy = math.floor(y / map.tileheight) + 1,
		ex = math.ceil((x + view.width) / map.tilewidth),
		ey = math.ceil((y + view.height) / map.tileheight),
	}
	hajtech.currentplayer = view.player
	map:draw(drawrange)

	love.graphics.pop()
	love.graphics.setCanvas(oldcanvas)
end

function viewport.centerOn(view, entity)
	local ex = math.floor(entity.x) + (entity.w / 2)
	local ey = math.floor(entity.y) + (entity.h / 2)

	view.x = ex - (view.width / 2)
	view.y = ey - (view.height / 2)
end

function viewport.follow(view)
	local entity = view.entity or view.character
	local distancex = entity.viewport_window_x or 50
	local distancey = entity.viewport_window_y or 40
	local ex = math.floor(entity.x) + (entity.w / 2)
	local ey = math.floor(entity.y) + (entity.h / 2)

	local vx = ex - (view.width / 2)
	local vy = ey - (view.height / 2)

	local dx = vx - view.x
	local dy = vy - view.y

	local mx = sign(dx) * math.max(math.abs(dx) - distancex, 0)
	local my = sign(dy) * math.max(math.abs(dy) - distancey, 0)

	view.x = view.x + mx
	view.y = view.y + my
end

return viewport
